@@ -183,8 +183,6 @@ class ContractState(StateBase):
         Returns:
              dict:
         """
-        codejson = self.Code.ToJson()
-
         name = 'Contract'
 
         try:
@@ -192,10 +190,11 @@ class ContractState(StateBase):
         except Exception as e:
             pass
 
-        jsn = {
+        jsn = {'version': self.StateVersion}
 
-            'version': self.StateVersion,
-            'code': codejson,
+        jsn_code = self.Code.ToJson()
+
+        jsn_contract = {
             'name': name,
             'code_version': self.CodeVersion.decode('utf-8'),
             'author': self.Author.decode('utf-8'),
@@ -208,6 +207,9 @@ class ContractState(StateBase):
             }
         }
 
+        jsn.update(jsn_code)
+        jsn.update(jsn_contract)
+
         if self._nep_token:
             jsn['token'] = self._nep_token.ToJson()
 
