@@ -6,7 +6,7 @@ import os
 class LevelDBTest(BlockchainFixtureTestCase):
 
     @classmethod
-    def leveldb_testpath(self):
+    def leveldb_testpath(cls):
         return os.path.join(settings.DATA_DIR_PATH, 'fixtures/test_chain')
 
     # test need to be updated whenever we change the fixtures
