@@ -1,9 +1,12 @@
 import binascii
 import random
+import datetime
 from twisted.internet.protocol import Protocol
 from twisted.internet import error as twisted_error
-from twisted.internet import reactor, task, threads
-
+from twisted.internet import reactor, task, defer
+from twisted.internet.address import IPv4Address
+from twisted.internet.defer import CancelledError
+from twisted.internet import error
 from neo.Core.Blockchain import Blockchain as BC
 from neocore.IO.BinaryReader import BinaryReader
 from neo.Network.Message import Message
@@ -19,28 +22,108 @@ from .Payloads.AddrPayload import AddrPayload
 from .InventoryType import InventoryType
 from neo.Settings import settings
 from neo.logging import log_manager
+from neo.Network.address import Address
 
 logger = log_manager.getLogger('network')
-
+logger_verbose = log_manager.getLogger('network.verbose')
 MODE_MAINTAIN = 7
 MODE_CATCHUP = 2
 
+mode_to_name = {MODE_CATCHUP: 'CATCHUP', MODE_MAINTAIN: 'MAINTAIN'}
+
+HEARTBEAT_BLOCKS = 'B'
+HEARTBEAT_HEADERS = 'H'
+
 
 class NeoNode(Protocol):
     Version = None
 
     leader = None
 
-    block_loop = None
-    block_loop_deferred = None
-
-    peer_loop = None
-    peer_loop_deferred = None
-
-    sync_mode = MODE_CATCHUP
-
     identifier = None
 
+    def has_tasks_running(self):
+        block = False
+        header = False
+        peer = False
+        if self.block_loop and self.block_loop.running:
+            block = True
+
+        if self.peer_loop and self.peer_loop.running:
+            peer = True
+
+        if self.header_loop and self.header_loop.running:
+            header = True
+
+        return block and header and peer
+
+    def start_all_tasks(self):
+        if not self.disconnecting:
+            self.start_block_loop()
+            self.start_header_loop()
+            self.start_peerinfo_loop()
+
+    def start_block_loop(self):
+        logger_verbose.debug(f"{self.prefix} start_block_loop")
+        if self.block_loop and self.block_loop.running:
+            logger_verbose.debug(f"start_block_loop: still running -> stopping...")
+            self.stop_block_loop()
+        self.block_loop = task.LoopingCall(self.AskForMoreBlocks)
+        self.block_loop_deferred = self.block_loop.start(self.sync_mode, now=False)
+        self.block_loop_deferred.addErrback(self.OnLoopError)
+        # self.leader.task_handles[self.block_loop] = self.prefix + f"{'block_loop':>15}"
+
+    def stop_block_loop(self, cancel=True):
+        logger_verbose.debug(f"{self.prefix} stop_block_loop: cancel -> {cancel}")
+        if self.block_loop:
+            logger_verbose.debug(f"{self.prefix} self.block_loop true")
+            if self.block_loop.running:
+                logger_verbose.debug(f"{self.prefix} stop_block_loop, calling stop")
+                self.block_loop.stop()
+        if cancel and self.block_loop_deferred:
+            logger_verbose.debug(f"{self.prefix} stop_block_loop: trying to cancel")
+            self.block_loop_deferred.cancel()
+
+    def start_peerinfo_loop(self):
+        logger_verbose.debug(f"{self.prefix} start_peerinfo_loop")
+        if self.peer_loop and self.peer_loop.running:
+            logger_verbose.debug(f"start_peer_loop: still running -> stopping...")
+            self.stop_peerinfo_loop()
+        self.peer_loop = task.LoopingCall(self.RequestPeerInfo)
+        self.peer_loop_deferred = self.peer_loop.start(120, now=False)
+        self.peer_loop_deferred.addErrback(self.OnLoopError)
+        # self.leader.task_handles[self.peer_loop] = self.prefix + f"{'peerinfo_loop':>15}"
+
+    def stop_peerinfo_loop(self, cancel=True):
+        logger_verbose.debug(f"{self.prefix} stop_peerinfo_loop: cancel -> {cancel}")
+        if self.peer_loop and self.peer_loop.running:
+            logger_verbose.debug(f"{self.prefix} stop_peerinfo_loop, calling stop")
+            self.peer_loop.stop()
+        if cancel and self.peer_loop_deferred:
+            logger_verbose.debug(f"{self.prefix} stop_peerinfo_loop: trying to cancel")
+            self.peer_loop_deferred.cancel()
+
+    def start_header_loop(self):
+        logger_verbose.debug(f"{self.prefix} start_header_loop")
+        if self.header_loop and self.header_loop.running:
+            logger_verbose.debug(f"start_header_loop: still running -> stopping...")
+            self.stop_header_loop()
+        self.header_loop = task.LoopingCall(self.AskForMoreHeaders)
+        self.header_loop_deferred = self.header_loop.start(5, now=False)
+        self.header_loop_deferred.addErrback(self.OnLoopError)
+        # self.leader.task_handles[self.header_loop] = self.prefix + f"{'header_loop':>15}"
+
+    def stop_header_loop(self, cancel=True):
+        logger_verbose.debug(f"{self.prefix} stop_header_loop: cancel -> {cancel}")
+        if self.header_loop:
+            logger_verbose.debug(f"{self.prefix} self.header_loop true")
+            if self.header_loop.running:
+                logger_verbose.debug(f"{self.prefix} stop_header_loop, calling stop")
+                self.header_loop.stop()
+        if cancel and self.header_loop_deferred:
+            logger_verbose.debug(f"{self.prefix} stop_header_loop: trying to cancel")
+            self.header_loop_deferred.cancel()
+
     def __init__(self, incoming_client=False):
         """
         Create an instance.
@@ -55,32 +138,63 @@ class NeoNode(Protocol):
         self.nodeid = self.leader.NodeId
         self.remote_nodeid = random.randint(1294967200, 4294967200)
         self.endpoint = ''
+        self.address = None
         self.buffer_in = bytearray()
         self.myblockrequests = set()
         self.bytes_in = 0
         self.bytes_out = 0
 
+        self.sync_mode = MODE_CATCHUP
+
         self.host = None
         self.port = None
-        self.identifier = self.leader.NodeCount
-        self.leader.NodeCount += 1
+
         self.incoming_client = incoming_client
+        self.handshake_complete = False
         self.expect_verack_next = False
+        self.start_outstanding_data_request = {HEARTBEAT_BLOCKS: 0, HEARTBEAT_HEADERS: 0}
 
-        self.Log("New Node created %s " % self.identifier)
+        self.block_loop = None
+        self.block_loop_deferred = None
 
-    def Disconnect(self, reason=None):
+        self.peer_loop = None
+        self.peer_loop_deferred = None
+
+        self.header_loop = None
+        self.header_loop_deferred = None
+
+        self.disconnect_deferred = None
+        self.disconnecting = False
+
+        logger.debug(f"{self.prefix} new node created, not yet connected")
+
+    def Disconnect(self, reason=None, isDead=True):
         """Close the connection with the remote node client."""
+        self.disconnecting = True
         self.expect_verack_next = False
         if reason:
-            logger.debug(reason)
-        self.transport.loseConnection()
+            logger.debug(f"Disconnecting with reason: {reason}")
+        self.stop_block_loop()
+        self.stop_header_loop()
+        self.stop_peerinfo_loop()
+        if isDead:
+            self.leader.AddDeadAddress(self.address, reason=f"{self.prefix} Forced disconnect by us")
+
+        self.leader.forced_disconnect_by_us += 1
+
+        self.disconnect_deferred = defer.Deferred()
+        self.disconnect_deferred.debug = True
+        # force disconnection without waiting on the other side
+        # calling later to give func caller time to add callbacks to the deferred
+        reactor.callLater(1, self.transport.abortConnection)
+        return self.disconnect_deferred
 
     @property
-    def Address(self):
-        if self.endpoint:
-            return "%s:%s" % (self.endpoint.host, self.endpoint.port)
-        return ""
+    def prefix(self):
+        if isinstance(self.endpoint, IPv4Address) and self.identifier is not None:
+            return f"[{self.identifier:03}][{mode_to_name[self.sync_mode]}][{self.address:>21}]"
+        else:
+            return f""
 
     def Name(self):
         """
@@ -89,7 +203,10 @@ class NeoNode(Protocol):
         Returns:
             str:
         """
-        return self.transport.getPeer()
+        name = ""
+        if self.Version:
+            name = self.Version.UserAgent
+        return name
 
     def GetNetworkAddressWithTime(self):
         """
@@ -113,15 +230,28 @@ class NeoNode(Protocol):
         biM = self.bytes_in / 1000000  # megabyes
         boM = self.bytes_out / 1000000
 
-        return "%s MB in / %s MB out" % (biM, boM)
+        return f"{biM:>10} MB in / {boM:>10} MB out"
 
     def connectionMade(self):
         """Callback handler from twisted when establishing a new connection."""
         self.endpoint = self.transport.getPeer()
+        # get the reference to the Address object in NodeLeader so we can manipulate it properly.
+        tmp_addr = Address(f"{self.endpoint.host}:{self.endpoint.port}")
+        try:
+            known_idx = self.leader.KNOWN_ADDRS.index(tmp_addr)
+            self.address = self.leader.KNOWN_ADDRS[known_idx]
+        except ValueError:
+            # Not found.
+            self.leader.AddKnownAddress(tmp_addr)
+            self.address = tmp_addr
+
+        self.address.address = "%s:%s" % (self.endpoint.host, self.endpoint.port)
         self.host = self.endpoint.host
         self.port = int(self.endpoint.port)
         self.leader.AddConnectedPeer(self)
-        self.Log("Connection from %s" % self.endpoint)
+        self.leader.RemoveFromQueue(self.address)
+        self.leader.peers_connecting -= 1
+        logger.debug(f"{self.address} connection established")
         if self.incoming_client:
             # start protocol
             self.SendVersion()
@@ -129,28 +259,57 @@ class NeoNode(Protocol):
     def connectionLost(self, reason=None):
         """Callback handler from twisted when a connection was lost."""
         try:
-            if self.block_loop_deferred:
-                self.block_loop_deferred.cancel()
-            if self.peer_loop_deferred:
-                self.peer_loop_deferred.cancel()
-
-            if self.block_loop:
-                if self.block_loop.running:
-                    self.block_loop.stop()
-            if self.peer_loop:
-                if self.peer_loop.running:
-                    self.peer_loop.stop()
+            self.connected = False
+            self.stop_block_loop()
+            self.stop_peerinfo_loop()
+            self.stop_header_loop()
 
             self.ReleaseBlockRequests()
             self.leader.RemoveConnectedPeer(self)
 
+            time_expired = self.time_expired(HEARTBEAT_BLOCKS)
+            # some NEO-cli versions have a 30s timeout to receive block/consensus or tx messages. By default neo-python doesn't respond to these requests
+            if time_expired > 20:
+                self.address.last_connection = Address.Now()
+                self.leader.AddDeadAddress(self.address, reason=f"{self.prefix} Premature disconnect")
+
             if reason and reason.check(twisted_error.ConnectionDone):
-                self.Log("client {} disconnected normally with reason:{}".format(self.remote_nodeid, reason))
+                # this might happen if they close our connection because they've reached max peers or something similar
+                logger.debug(f"{self.prefix} disconnected normally with reason:{reason.value}")
+                self._check_for_consecutive_disconnects("connection done")
+
+            elif reason and reason.check(twisted_error.ConnectionLost):
+                # Can be due to a timeout. Only if this happened again within 5 minutes do we label the node as bad
+                # because then it clearly doesn't want to talk to us or we have a bad connection to them.
+                # Otherwise allow for the node to be queued again by NodeLeader.
+                logger.debug(f"{self.prefix} disconnected with connectionlost reason: {reason.value}")
+                self._check_for_consecutive_disconnects("connection lost")
+
             else:
-                self.Log("%s disconnected %s" % (self.remote_nodeid, reason))
+                logger.debug(f"{self.prefix} disconnected with reason: {reason.value}")
         except Exception as e:
             logger.error("Error with connection lost: %s " % e)
 
+        def try_me(err):
+            err.check(error.ConnectionAborted)
+
+        if self.disconnect_deferred:
+            d, self.disconnect_deferred = self.disconnect_deferred, None  # type: defer.Deferred
+            d.addErrback(try_me)
+            if len(d.callbacks) > 0:
+                d.callback(reason)
+            else:
+                print("connLost, disconnect_deferred cancelling!")
+                d.cancel()
+
+    def _check_for_consecutive_disconnects(self, error_name):
+        now = datetime.datetime.utcnow().timestamp()
+        FIVE_MINUTES = 5 * 60
+        if self.address.last_connection != 0 and now - self.address.last_connection < FIVE_MINUTES:
+            self.leader.AddDeadAddress(self.address, reason=f"{self.prefix} second {error_name} within 5 minutes")
+        else:
+            self.address.last_connection = Address.Now()
+
     def ReleaseBlockRequests(self):
         bcr = BC.Default().BlockRequests
         requests = self.myblockrequests
@@ -160,7 +319,7 @@ class NeoNode(Protocol):
                 if req in bcr:
                     bcr.remove(req)
             except Exception as e:
-                self.Log("Could not remove request %s " % e)
+                logger.debug(f"{self.prefix} Could not remove request {e}")
 
         self.myblockrequests = set()
 
@@ -199,7 +358,8 @@ class NeoNode(Protocol):
                 return False
 
         except Exception as e:
-            self.Log('Error: Could not read initial bytes %s ' % e)
+            logger.debug(f"{self.prefix} Error: could not read message header from stream {e}")
+            # self.Log('Error: Could not read initial bytes %s ' % e)
             return False
 
         finally:
@@ -227,7 +387,8 @@ class NeoNode(Protocol):
             self.MessageReceived(message)
 
         except Exception as e:
-            self.Log('Error: Could not extract message: %s ' % e)
+            logger.debug(f"{self.prefix} Could not extract message {e}")
+            # self.Log('Error: Could not extract message: %s ' % e)
             return False
 
         finally:
@@ -242,7 +403,6 @@ class NeoNode(Protocol):
         Args:
             m (neo.Network.Message):
         """
-
         if m.Command == 'verack':
             # only respond with a verack when we connect to another client, not when a client connected to us or
             # we might end up in a verack loop
@@ -266,33 +426,45 @@ class NeoNode(Protocol):
         elif m.Command == 'getheaders':
             self.HandleGetHeadersMessageReceived(m.Payload)
         elif m.Command == 'headers':
-            reactor.callFromThread(self.HandleBlockHeadersReceived, m.Payload)
-
+            self.HandleBlockHeadersReceived(m.Payload)
         elif m.Command == 'addr':
             self.HandlePeerInfoReceived(m.Payload)
         else:
-            self.Log("Command not implemented {} {} ".format(m.Command, self.endpoint))
+            logger.debug(f"{self.prefix} Command not implemented: {m.Command}")
 
     def OnLoopError(self, err):
-        self.Log("On neo Node loop error %s " % err)
+        # happens if we cancel the disconnect_deferred before it is executed
+        # causes no harm
+        if type(err.value) == CancelledError:
+            logger_verbose.debug(f"{self.prefix} OnLoopError cancelled deferred")
+            return
+        logger.debug(f"{self.prefix} On neo Node loop error {err}")
 
     def onThreadDeferredErr(self, err):
-        logger.error("On Call from thread error %s " % err)
+        if type(err.value) == CancelledError:
+            logger_verbose.debug(f"{self.prefix} onThreadDeferredError cancelled deferred")
+            return
+        logger.debug(f"{self.prefix} On Call from thread error {err}")
+
+    def keep_alive(self):
+        ka = Message("ping")
+        self.SendSerializedMessage(ka)
 
     def ProtocolReady(self):
-        self.block_loop = task.LoopingCall(self.AskForMoreBlocks)
-        self.block_loop_deferred = self.block_loop.start(self.sync_mode, now=False)
-        self.block_loop_deferred.addErrback(self.OnLoopError)
-
-        # ask every 2 minutes for new peers
-        self.peer_loop = task.LoopingCall(self.RequestPeerInfo)
-        self.peer_loop_deferred = self.peer_loop.start(120, now=False)
-        self.peer_loop_deferred.addErrback(self.OnLoopError)
+        # do not start the looping tasks if we're in the BlockRequests catchup task
+        # otherwise BCRLen will not drop because the new node will continue adding blocks
+        logger_verbose.debug(f"{self.prefix} ProtocolReady called")
+        if not self.leader.check_bcr_loop or (self.leader.check_bcr_loop and not self.leader.check_bcr_loop.running):
+            logger_verbose.debug(f"{self.prefix} Protocol ready -> starting loops")
+            self.start_block_loop()
+            self.start_peerinfo_loop()
+            self.start_header_loop()
 
         self.RequestPeerInfo()
-        self.AskForMoreHeaders()
 
     def AskForMoreHeaders(self):
+        logger.debug(f"{self.prefix} asking for more headers, starting from {BC.Default().HeaderHeight}")
+        self.health_check(HEARTBEAT_HEADERS)
         get_headers_message = Message("getheaders", GetBlocksPayload(hash_start=[BC.Default().CurrentHeaderHash]))
         self.SendSerializedMessage(get_headers_message)
 
@@ -308,18 +480,16 @@ class NeoNode(Protocol):
             self.sync_mode = MODE_MAINTAIN
 
         if self.sync_mode != current_mode:
-            if self.block_loop and self.block_loop.running:
-                self.block_loop.stop()
-            self.block_loop_deferred.cancel()
-            self.block_loop = task.LoopingCall(self.AskForMoreBlocks)
-            self.block_loop_deferred = self.block_loop.start(self.sync_mode)
-            self.block_loop_deferred.addErrback(self.OnLoopError)
+            logger.debug(f"{self.prefix} changing sync_mode to {mode_to_name[self.sync_mode]}")
+            self.stop_block_loop()
+            self.start_block_loop()
 
         else:
             if len(BC.Default().BlockRequests) > self.leader.BREQMAX:
-                self.leader.ResetBlockRequestsAndCache()
-
-            self.DoAskForMoreBlocks()
+                logger.debug(f"{self.prefix} data request speed exceeding node response rate...pausing to catch up")
+                self.leader.throttle_sync()
+            else:
+                self.DoAskForMoreBlocks()
 
     def DoAskForMoreBlocks(self):
         hashes = []
@@ -331,7 +501,7 @@ class NeoNode(Protocol):
             do_go_ahead = True
 
         first = None
-        while hashstart < current_header_height and len(hashes) < self.leader.BREQPART:
+        while hashstart <= current_header_height and len(hashes) < self.leader.BREQPART:
             hash = BC.Default().GetHeaderHash(hashstart)
             if not do_go_ahead:
                 if hash is not None and hash not in BC.Default().BlockRequests \
@@ -343,23 +513,26 @@ class NeoNode(Protocol):
                     self.myblockrequests.add(hash)
                     hashes.append(hash)
             else:
-                if not first:
-                    first = hashstart
-                BC.Default().BlockRequests.add(hash)
-                self.myblockrequests.add(hash)
-                hashes.append(hash)
+                if hash is not None:
+                    if not first:
+                        first = hashstart
+                    BC.Default().BlockRequests.add(hash)
+                    self.myblockrequests.add(hash)
+                    hashes.append(hash)
 
             hashstart += 1
 
-        self.Log("asked for more blocks ... %s thru %s (%s blocks) stale count %s BCRLen: %s " % (first, hashstart, len(hashes), BC.Default().BlockSearchTries, len(BC.Default().BlockRequests)))
-
         if len(hashes) > 0:
+            logger.debug(
+                f"{self.prefix} asking for more blocks {first} - {hashstart} ({len(hashes)}) stale count: {BC.Default().BlockSearchTries} "
+                f"BCRLen: {len(BC.Default().BlockRequests)}")
+            self.health_check(HEARTBEAT_BLOCKS)
             message = Message("getdata", InvPayload(InventoryType.Block, hashes))
-            sendDeferred = threads.deferToThread(self.SendSerializedMessage, message)
-            sendDeferred.addErrback(self.onThreadDeferredErr)
+            self.SendSerializedMessage(message)
 
     def RequestPeerInfo(self):
         """Request the peer address information from the remote client."""
+        logger.debug(f"{self.prefix} requesting peer info")
         self.SendSerializedMessage(Message('getaddr'))
 
     def HandlePeerInfoReceived(self, payload):
@@ -369,19 +542,20 @@ class NeoNode(Protocol):
         if not addrs:
             return
 
-        for index, nawt in enumerate(addrs.NetworkAddressesWithTime):
-            self.leader.RemoteNodePeerReceived(nawt.Address, nawt.Port, index)
+        for nawt in addrs.NetworkAddressesWithTime:
+            self.leader.RemoteNodePeerReceived(nawt.Address, nawt.Port, self.prefix)
 
     def SendPeerInfo(self):
-        if not self.leader.ServiceEnabled:
-            return
+        # if not self.leader.ServiceEnabled:
+        #     return
 
         peerlist = []
         for peer in self.leader.Peers:
             addr = peer.GetNetworkAddressWithTime()
             if addr is not None:
                 peerlist.append(addr)
-        self.Log("Peer list %s " % list(map(lambda p: p.ToString(), peerlist)))
+        peer_str_list = list(map(lambda p: p.ToString(), peerlist))
+        logger.debug(f"{self.prefix} Sending Peer list {peer_str_list}")
 
         addrpayload = AddrPayload(addresses=peerlist)
         message = Message('addr', addrpayload)
@@ -422,6 +596,10 @@ class NeoNode(Protocol):
         """Handle the `verack` response."""
         m = Message('verack')
         self.SendSerializedMessage(m)
+        self.leader.NodeCount += 1
+        self.identifier = self.leader.NodeCount
+        logger.debug(f"{self.prefix} Handshake complete!")
+        self.handshake_complete = True
         self.ProtocolReady()
 
     def HandleInvMessage(self, payload):
@@ -449,7 +627,6 @@ class NeoNode(Protocol):
                     BC.Default().BlockRequests.add(hash)
                     self.myblockrequests.add(hash)
             if len(ok_hashes):
-                #                logger.info("OK HASHES, get data %s " % ok_hashes)
                 message = Message("getdata", InvPayload(InventoryType.Block, ok_hashes))
                 self.SendSerializedMessage(message)
 
@@ -471,7 +648,7 @@ class NeoNode(Protocol):
             self.bytes_out += len(ba2)
             self.transport.write(ba2)
         except Exception as e:
-            self.Log("Could not send serialized message %s " % e)
+            logger.debug(f"Could not send serialized message {e}")
 
     def HandleBlockHeadersReceived(self, inventory):
         """
@@ -483,16 +660,12 @@ class NeoNode(Protocol):
         try:
             inventory = IOHelper.AsSerializableWithType(inventory, 'neo.Network.Payloads.HeadersPayload.HeadersPayload')
             if inventory is not None:
+                logger.debug(f"{self.prefix} received headers")
+                self.heart_beat(HEARTBEAT_HEADERS)
                 BC.Default().AddHeaders(inventory.Headers)
 
-            if BC.Default().HeaderHeight < self.Version.StartHeight:
-                self.AskForMoreHeaders()
-            else:
-                deferredCall = task.deferLater(reactor, 5, self.AskForMoreHeaders)
-                deferredCall.addErrback(self.onThreadDeferredErr)
-
         except Exception as e:
-            self.Log("Error handling Block headers %s " % e)
+            logger.debug(f"Error handling Block headers {e}")
 
     def HandleBlockReceived(self, inventory):
         """
@@ -506,13 +679,75 @@ class NeoNode(Protocol):
             return
 
         blockhash = block.Hash.ToBytes()
-        if blockhash in BC.Default().BlockRequests:
-            BC.Default().BlockRequests.remove(blockhash)
-        if blockhash in self.myblockrequests:
-            self.myblockrequests.remove(blockhash)
-
+        try:
+            if blockhash in BC.Default().BlockRequests:
+                BC.Default().BlockRequests.remove(blockhash)
+        except KeyError:
+            pass
+        try:
+            if blockhash in self.myblockrequests:
+                # logger.debug(f"{self.prefix} received block: {block.Index}")
+                self.heart_beat(HEARTBEAT_BLOCKS)
+                self.myblockrequests.remove(blockhash)
+        except KeyError:
+            pass
         self.leader.InventoryReceived(block)
 
+    def time_expired(self, what):
+        now = datetime.datetime.utcnow().timestamp()
+        start_time = self.start_outstanding_data_request.get(what)
+        if start_time == 0:
+            delta = 0
+        else:
+            delta = now - start_time
+        return delta
+
+    def health_check(self, what):
+        # now = datetime.datetime.utcnow().timestamp()
+        # delta = now - self.start_outstanding_data_request.get(what)
+
+        time_expired = self.time_expired(what)
+
+        if time_expired == 0:
+            # startup scenario, just go
+            logger.debug(f"{self.prefix}[HEALTH][{what}] startup or bcr catchup heart_beat")
+            self.heart_beat(what)
+        else:
+            if self.sync_mode == MODE_CATCHUP:
+                response_threshold = 45  # seconds
+            else:
+                response_threshold = 90  #
+            if time_expired > response_threshold:
+                header_time = self.time_expired(HEARTBEAT_HEADERS)
+                header_bad = header_time > response_threshold
+                block_time = self.time_expired(HEARTBEAT_BLOCKS)
+                blocks_bad = block_time > response_threshold
+                if header_bad and blocks_bad:
+                    logger.debug(
+                        f"{self.prefix}[HEALTH] FAILED - No response for Headers {header_time:.2f} and Blocks {block_time:.2f} seconds. Removing node...")
+                    self.Disconnect()
+                elif blocks_bad and self.leader.check_bcr_loop and self.leader.check_bcr_loop.running:
+                    # when we're in data throttling it is never acceptable if blocks don't come in. 
+                    logger.debug(
+                        f"{self.prefix}[HEALTH] FAILED - No Blocks for {block_time:.2f} seconds while throttling. Removing node...")
+                    self.Disconnect()
+                else:
+                    if header_bad:
+                        logger.debug(
+                            f"{self.prefix}[HEALTH] Headers FAILED @ {header_time:.2f}s, but Blocks OK @ {block_time:.2f}s. Keeping node...")
+                    else:
+                        logger.debug(
+                            f"{self.prefix}[HEALTH] Headers OK @ {header_time:.2f}s, but Blocks FAILED @ {block_time:.2f}s. Keeping node...")
+
+                # logger.debug(
+                #     f"{self.prefix}[HEALTH][{what}] FAILED - No response for {time_expired:.2f} seconds. Removing node...")
+
+            else:
+                logger.debug(f"{self.prefix}[HEALTH][{what}] OK - response time {time_expired:.2f}")
+
+    def heart_beat(self, what):
+        self.start_outstanding_data_request[what] = datetime.datetime.utcnow().timestamp()
+
     def HandleGetHeadersMessageReceived(self, payload):
 
         if not self.leader.ServiceEnabled:
@@ -528,7 +763,7 @@ class NeoNode(Protocol):
         hash = inventory.HashStart[0]
 
         if hash is None or hash == inventory.HashStop:
-            self.Log("getheaders: Hash {} not found or hashstop reached".format(inventory.HashStart))
+            logger.debug("getheaders: Hash {} not found or hashstop reached".format(inventory.HashStart))
             return
 
         headers = []
@@ -546,7 +781,7 @@ class NeoNode(Protocol):
 
     def HandleBlockReset(self, hash):
         """Process block reset request."""
-        self.myblockrequests = []
+        self.myblockrequests = set()
 
     def HandleGetDataMessageReceived(self, payload):
         """
@@ -634,5 +869,8 @@ class NeoNode(Protocol):
 
         return True
 
-    def Log(self, msg):
-        logger.debug("[%s][mode %s] %s - %s" % (self.identifier, self.sync_mode, self.endpoint, msg))
+    def __eq__(self, other):
+        if type(other) is type(self):
+            return self.address == other.address and self.identifier == other.identifier
+        else:
+            return False
