@@ -4,8 +4,7 @@ from neo.Blockchain import GetBlockchain
 from neo.VM.ScriptBuilder import ScriptBuilder
 from neo.VM.InteropService import InteropInterface
 from neo.Network.NodeLeader import NodeLeader
-from neo.Prompt.Utils import parse_param, get_asset_attachments, lookup_addr_str, get_owners_from_params, gather_param, get_parse_addresses
-
+from neo.Prompt import Utils as PromptUtils
 from neo.Implementations.Blockchains.LevelDB.DBCollection import DBCollection
 from neo.Implementations.Blockchains.LevelDB.DBPrefix import DBPrefix
 from neo.Implementations.Blockchains.LevelDB.CachedScriptTable import CachedScriptTable
@@ -35,6 +34,7 @@ from neo.EventHub import events
 from prompt_toolkit import prompt
 from copy import deepcopy
 from neo.logging import log_manager
+from neo.Prompt.PromptPrinter import prompt_print as print
 
 logger = log_manager.getLogger()
 
@@ -47,9 +47,13 @@ DEFAULT_MIN_FEE = Fixed8.FromDecimal(.0001)
 
 def InvokeContract(wallet, tx, fee=Fixed8.Zero(), from_addr=None, owners=None):
     if from_addr is not None:
-        from_addr = lookup_addr_str(wallet, from_addr)
+        from_addr = PromptUtils.lookup_addr_str(wallet, from_addr)
 
-    wallet_tx = wallet.MakeTransaction(tx=tx, fee=fee, use_standard=True, from_addr=from_addr)
+    try:
+        wallet_tx = wallet.MakeTransaction(tx=tx, fee=fee, use_standard=True, from_addr=from_addr)
+    except ValueError:
+        print("Insufficient funds")
+        return False
 
     if wallet_tx:
 
@@ -93,7 +97,11 @@ def InvokeContract(wallet, tx, fee=Fixed8.Zero(), from_addr=None, owners=None):
 
 
 def InvokeWithTokenVerificationScript(wallet, tx, token, fee=Fixed8.Zero(), invoke_attrs=None):
-    wallet_tx = wallet.MakeTransaction(tx=tx, fee=fee, use_standard=True)
+    try:
+        wallet_tx = wallet.MakeTransaction(tx=tx, fee=fee, use_standard=True)
+    except ValueError:
+        print("Insufficient funds")
+        return False
 
     if wallet_tx:
 
@@ -158,16 +166,16 @@ def TestInvokeContract(wallet, args, withdrawal_tx=None,
         #
         params = args[1:] if len(args) > 1 else []
 
-        params, neo_to_attach, gas_to_attach = get_asset_attachments(params)
-        params, parse_addresses = get_parse_addresses(params)
+        params, neo_to_attach, gas_to_attach = PromptUtils.get_asset_attachments(params)
+        params, parse_addresses = PromptUtils.get_parse_addresses(params)
         params.reverse()
 
         if '--i' in params:
             params = []
             for index, iarg in enumerate(contract.Code.ParameterList):
-                param, abort = gather_param(index, iarg)
+                param, abort = PromptUtils.gather_param(index, iarg)
                 if abort:
-                    return None, None, None, None
+                    return None, None, None, None, False
                 params.append(param)
             params.reverse()
 
@@ -176,18 +184,18 @@ def TestInvokeContract(wallet, args, withdrawal_tx=None,
         for p in params:
 
             if parse_params:
-                item = parse_param(p, wallet, parse_addr=parse_addresses)
+                item = PromptUtils.parse_param(p, wallet, parse_addr=parse_addresses)
             else:
                 item = p
             if type(item) is list:
                 item.reverse()
                 listlength = len(item)
                 for listitem in item:
-                    subitem = parse_param(listitem, wallet, parse_addr=parse_addresses)
+                    subitem = PromptUtils.parse_param(listitem, wallet, parse_addr=parse_addresses)
                     if type(subitem) is list:
                         subitem.reverse()
                         for listitem2 in subitem:
-                            subsub = parse_param(listitem2, wallet, parse_addr=parse_addresses)
+                            subsub = PromptUtils.parse_param(listitem2, wallet, parse_addr=parse_addresses)
                             sb.push(subsub)
                         sb.push(len(subitem))
                         sb.Emit(PACK)
@@ -225,7 +233,7 @@ def TestInvokeContract(wallet, args, withdrawal_tx=None,
 
         print("Contract %s not found" % args[0])
 
-    return None, None, None, None
+    return None, None, None, None, False
 
 
 def make_unique_script_attr(attributes):
@@ -262,7 +270,7 @@ def test_invoke(script, wallet, outputs, withdrawal_tx=None,
     # print("invoke script %s " % script)
 
     if from_addr is not None:
-        from_addr = lookup_addr_str(wallet, from_addr)
+        from_addr = PromptUtils.lookup_addr_str(wallet, from_addr)
 
     bc = GetBlockchain()
 
@@ -300,7 +308,10 @@ def test_invoke(script, wallet, outputs, withdrawal_tx=None,
     if withdrawal_tx is not None:
         wallet_tx = tx
     else:
-        wallet_tx = wallet.MakeTransaction(tx=tx, from_addr=from_addr)
+        try:
+            wallet_tx = wallet.MakeTransaction(tx=tx, from_addr=from_addr)
+        except ValueError:
+            pass
 
     context = ContractParametersContext(wallet_tx)
     wallet.Sign(context)
@@ -317,7 +328,7 @@ def test_invoke(script, wallet, outputs, withdrawal_tx=None,
     if context.Completed:
         wallet_tx.scripts = context.GetScripts()
     else:
-        logger.warn("Not gathering signatures for test build.  For a non-test invoke that would occur here.")
+        logger.warning("Not gathering signatures for test build.  For a non-test invoke that would occur here.")
     #        if not gather_signatures(context, wallet_tx, owners):
     #            return None, [], 0, None
 
@@ -368,18 +379,18 @@ def test_invoke(script, wallet, outputs, withdrawal_tx=None,
             wallet_tx.outputs = outputs
             wallet_tx.Attributes = [] if invoke_attrs is None else deepcopy(invoke_attrs)
 
-            return wallet_tx, net_fee, engine.ResultStack.Items, engine.ops_processed
+            return wallet_tx, net_fee, engine.ResultStack.Items, engine.ops_processed, success
 
         # this allows you to to test invocations that fail
         else:
             wallet_tx.outputs = outputs
             wallet_tx.Attributes = [] if invoke_attrs is None else deepcopy(invoke_attrs)
-            return wallet_tx, min_fee, [], engine.ops_processed
+            return wallet_tx, min_fee, [], engine.ops_processed, success
 
     except Exception as e:
         service.ExecutionCompleted(engine, False, e)
 
-    return None, None, None, None
+    return None, None, None, None, False
 
 
 def test_deploy_and_invoke(deploy_script, invoke_args, wallet,
@@ -406,9 +417,13 @@ def test_deploy_and_invoke(deploy_script, invoke_args, wallet,
     dtx.Script = binascii.unhexlify(deploy_script)
 
     if from_addr is not None:
-        from_addr = lookup_addr_str(wallet, from_addr)
+        from_addr = PromptUtils.lookup_addr_str(wallet, from_addr)
+
+    try:
+        dtx = wallet.MakeTransaction(tx=dtx, from_addr=from_addr)
+    except ValueError:
+        pass
 
-    dtx = wallet.MakeTransaction(tx=dtx, from_addr=from_addr)
     context = ContractParametersContext(dtx)
     wallet.Sign(context)
     dtx.scripts = context.GetScripts()
@@ -455,15 +470,15 @@ def test_deploy_and_invoke(deploy_script, invoke_args, wallet,
 
         shash = contract_state.Code.ScriptHash()
 
-        invoke_args, neo_to_attach, gas_to_attach = get_asset_attachments(invoke_args)
-        invoke_args, no_parse_addresses = get_parse_addresses(invoke_args)
+        invoke_args, neo_to_attach, gas_to_attach = PromptUtils.get_asset_attachments(invoke_args)
+        invoke_args, no_parse_addresses = PromptUtils.get_parse_addresses(invoke_args)
 
         invoke_args.reverse()
 
         if '--i' in invoke_args:
             invoke_args = []
             for index, iarg in enumerate(contract_state.Code.ParameterList):
-                param, abort = gather_param(index, iarg)
+                param, abort = PromptUtils.gather_param(index, iarg)
                 if abort:
                     return None, [], 0, None
                 else:
@@ -473,16 +488,16 @@ def test_deploy_and_invoke(deploy_script, invoke_args, wallet,
         sb = ScriptBuilder()
 
         for p in invoke_args:
-            item = parse_param(p, wallet, parse_addr=no_parse_addresses)
+            item = PromptUtils.parse_param(p, wallet, parse_addr=no_parse_addresses)
             if type(item) is list:
                 item.reverse()
                 listlength = len(item)
                 for listitem in item:
-                    subitem = parse_param(listitem, wallet, parse_addr=no_parse_addresses)
+                    subitem = PromptUtils.parse_param(listitem, wallet, parse_addr=no_parse_addresses)
                     if type(subitem) is list:
                         subitem.reverse()
                         for listitem2 in subitem:
-                            subsub = parse_param(listitem2, wallet, parse_addr=no_parse_addresses)
+                            subsub = PromptUtils.parse_param(listitem2, wallet, parse_addr=no_parse_addresses)
                             sb.push(subsub)
                         sb.push(len(subitem))
                         sb.Emit(PACK)
@@ -527,7 +542,10 @@ def test_deploy_and_invoke(deploy_script, invoke_args, wallet,
                                                        data=contract.ScriptHash))
             itx.Attributes = make_unique_script_attr(itx.Attributes)
 
-        itx = wallet.MakeTransaction(tx=itx, from_addr=from_addr)
+        try:
+            itx = wallet.MakeTransaction(tx=itx, from_addr=from_addr)
+        except ValueError:
+            pass
 
         context = ContractParametersContext(itx)
         wallet.Sign(context)
