@@ -99,7 +99,7 @@ class SettingsHolder:
     DEBUG_STORAGE_PATH = 'Chains/debugstorage'
 
     ACCEPT_INCOMING_PEERS = False
-    CONNECTED_PEER_MAX = 5
+    CONNECTED_PEER_MAX = 20
 
     SERVICE_ENABLED = True
 
@@ -107,6 +107,11 @@ class SettingsHolder:
 
     VERSION_NAME = "/NEO-PYTHON:%s/" % __version__
 
+    RPC_SERVER = None
+    REST_SERVER = None
+    DEFAULT_RPC_SERVER = 'neo.api.JSONRPC.JsonRpcApi.JsonRpcApi'
+    DEFAULT_REST_SERVER = 'neo.api.REST.RestApi.RestApi'
+
     # Logging settings
     log_level = None
     log_smart_contract_events = False
@@ -163,6 +168,16 @@ class SettingsHolder:
     # Setup methods
     def setup(self, config_file):
         """ Setup settings from a JSON config file """
+
+        def get_config_and_warn(key, default, abort=False):
+            value = config.get(key, None)
+            if not value:
+                print(f"Cannot find {key} in settings, using default value: {default}")
+                value = default
+                if abort:
+                    sys.exit(-1)
+            return value
+
         if not self.DATA_DIR_PATH:
             # Setup default data dir
             self.set_data_dir(None)
@@ -192,16 +207,17 @@ class SettingsHolder:
         self.URI_PREFIX = config['UriPrefix']
         self.ACCEPT_INCOMING_PEERS = config.get('AcceptIncomingPeers', False)
 
-        self.BOOTSTRAP_FILE = config['BootstrapFile']
-        self.NOTIF_BOOTSTRAP_FILE = config['NotificationBootstrapFile']
-
+        self.BOOTSTRAP_NAME = get_config_and_warn('BootstrapName', "mainnet")
+        self.BOOTSTRAP_LOCATIONS = get_config_and_warn('BootstrapFiles', "abort", abort=True)
         Helper.ADDRESS_VERSION = self.ADDRESS_VERSION
 
-        self.USE_DEBUG_STORAGE = config.get('DebugStorage', True)
+        self.USE_DEBUG_STORAGE = config.get('DebugStorage', False)
         self.DEBUG_STORAGE_PATH = config.get('DebugStoragePath', 'Chains/debugstorage')
         self.NOTIFICATION_DB_PATH = config.get('NotificationDataPath', 'Chains/notification_data')
-        self.SERVICE_ENABLED = config.get('ServiceEnabled', False)
+        self.SERVICE_ENABLED = config.get('ServiceEnabled', self.ACCEPT_INCOMING_PEERS)
         self.COMPILER_NEP_8 = config.get('CompilerNep8', False)
+        self.REST_SERVER = config.get('RestServer', self.DEFAULT_REST_SERVER)
+        self.RPC_SERVER = config.get('RPCServer', self.DEFAULT_RPC_SERVER)
 
     def setup_mainnet(self):
         """ Load settings from the mainnet JSON config file """
@@ -251,10 +267,11 @@ class SettingsHolder:
             os.makedirs(self.DATA_DIR_PATH)
 
     def set_max_peers(self, num_peers):
-        try:
-            self.CONNECTED_PEER_MAX = int(num_peers)
-        except Exception as e:
-            logger.error("Please supply an integer number for max peers")
+        maxpeers = int(num_peers)
+        if maxpeers > 0:
+            self.CONNECTED_PEER_MAX = maxpeers
+        else:
+            raise ValueError
 
     def set_log_smart_contract_events(self, is_enabled=True):
         self.log_smart_contract_events = is_enabled
@@ -313,8 +330,10 @@ class SettingsHolder:
         # Add a warning for migration purposes if we created a chain dir
         if warn_migration and ROOT_INSTALL_PATH != self.DATA_DIR_PATH:
             if os.path.exists(os.path.join(ROOT_INSTALL_PATH, 'Chains')):
-                logger.warning("[MIGRATION] You are now using the blockchain data at %s, but it appears you have existing data at %s/Chains" % (chain_path, ROOT_INSTALL_PATH))
-                logger.warning("[MIGRATION] If you would like to use your existing data, please move any data at %s/Chains to %s " % (ROOT_INSTALL_PATH, chain_path))
+                logger.warning("[MIGRATION] You are now using the blockchain data at %s, but it appears you have existing data at %s/Chains" % (
+                    chain_path, ROOT_INSTALL_PATH))
+                logger.warning(
+                    "[MIGRATION] If you would like to use your existing data, please move any data at %s/Chains to %s " % (ROOT_INSTALL_PATH, chain_path))
                 logger.warning("[MIGRATION] Or you can continue using your existing data by starting your script with the `--datadir=.` flag")
 
     def check_privatenet(self):
